<?php
/**
 * @package JTotal BUF Framework
 * @author JTotal
 * @copyright Copyright (c) 2005 - 2023 JTotal
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPLv2 or Later
*/

defined ('_JEXEC') or die();

use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Installer\Installer;
use Joomla\CMS\Language\Text;



if ( ! class_exists('pkg_bufInstallerScript'))
{
    class pkg_bufInstallerScript
    {
        static $adapter;
        static $current_version;
        static $extensions         = [];
        static $file_string;
        static $min_joomla_version = [3 => '3.9.0', 4 => '4.0'];
        static $min_php_version    = [3 => '7.4', 4 => '7.4'];
        static $name;
        static $new_manifest;
        static $package_name;
        static $previous_version;
        static $previous_joomla_version;

        public function postflight($type, $parent)
        {



            $app = Factory::getApplication();

            $src = $parent->getParent()->getPath('source');
            $manifest = $parent->getParent()->manifest;
            $plugins = $manifest->xpath('plugins/plugin');

            $metapackages = $manifest->xpath('metapackages/metapackage');


            foreach ($metapackages as $key => $metapackage)
            {




                $installer = new Installer();

                //META LIBRARY
                if((string)$metapackage->attributes()->type == "library"){
                    $name = (string)$metapackage->attributes()->libraryname;

                    $final_path = JPATH_LIBRARIES.'/jtlibs';
                    if (Folder::exists($final_path))
                    {
                    $app->enqueueMessage("BUF: ".$name." already installed... no need metapackage.", 'Message');
                    continue;
                    }

                    //INSTALLING
                    $current_path = $src.'/metapackages/'.$name;
                    $result = $installer->install($current_path);

                    if (!$result)
                    {
                        $app->enqueueMessage("BUF: error installing ".$name." metapackage.", 'error');
                        continue;
                    }
                }

                //META PLUGIN
                if((string)$metapackage->attributes()->type == "plugin"){



                    $name = (string)$metapackage->attributes()->plugin;
                    $group = (string)$metapackage->attributes()->group;
                    $plugin_info = $this->getPluginInfoByName($name, $group);


                    $final_path = JPATH_PLUGINS.'/'.$group.'/'.$name;

                    if (Folder::exists($final_path))
                    {

                        $app->enqueueMessage("BUF: ".$name." already installed... no need metapackage.", 'Message');
                        continue;
                    }


                    //INSTALLING
                    $current_path = $src.'/metapackages/'.$name;


                    $result = $installer->install($current_path);


                    if (!$result)
                    {

                        $app->enqueueMessage("BUF: error installing ".$name." metapackage.", 'error');
                        continue;
                    }else{

                        $this->activeInstalledPlugin($name, $group);
                        $app->enqueueMessage("BUF: ".$name." metapackage installed... please UPDATE.", 'Warning');
                    }

                }

            }


            //ENABLE AJAX PLUGIN
            $this->activeInstalledPlugin("bufajax", "ajax");

            $this->removeUpdateSite('Ajax - BUF Framework');
            //remove update site

            //remove old Files (J3 bootstrap etc)
            $this->removeOldFiles();

        }

        private function removeOldFiles(){

            //Bootstrap 4 files
            if (Folder::exists(JPATH_BASE . '/templates/buf/libs/bootstrap4')){
                Folder::delete(JPATH_BASE . '/templates/buf/libs/bootstrap4');
            }
            //FA 4 files
            if (Folder::exists(JPATH_BASE . '/templates/buf/css/font-awesome4')){
                Folder::delete(JPATH_BASE . '/templates/buf/css/font-awesome4');
            }

        }


        private function removeUpdateSite($name)
        {

            $db = Factory::getDbo();
            $query = $db->getQuery(true);
            $query->delete($db->quoteName('#__update_sites'));
            $query->where($db->quoteName('name') . ' = ' . $db->quote( $name ));


            $db->setQuery($query);

            return true;
        }

        private function getTemplateInfoByName($name)
        {

            $db = Factory::getDbo();
            $query = $db->getQuery(true);
            $query->select('*');
            $query->from($db->quoteName('#__template_styles'));
            $query->where($db->quoteName('client_id') . ' = 0');
            $query->where($db->quoteName('template') . ' = ' . $db->quote( $name ));

            $db->setQuery($query);

            return $db->loadObject();
        }

        private function activeInstalledPlugin($name, $group)
        {

            $db = Factory::getDBO();
            $query = $db->getQuery(true);
            $fields = array(
                $db->quoteName('enabled') . ' = 1'
            );

            $conditions = array(
                $db->quoteName('type') . ' = ' . $db->quote('plugin'),
                $db->quoteName('element') . ' = ' . $db->quote($name),
                $db->quoteName('folder') . ' = ' . $db->quote($group)
            );

            $query->update($db->quoteName('#__extensions'))->set($fields)->where($conditions);
            $db->setQuery($query);
            //db->execute();
        }

        private function getPluginInfoByName($name, $group)
        {

            $db = Factory::getDbo();
            $query = $db->getQuery(true);
            $query->select('*');
            $query->from($db->quoteName('#__extensions'));
            $query->where($db->quoteName('type') . ' = ' . $db->quote('plugin'));
            $query->where($db->quoteName('element') . ' = ' . $db->quote( $name ));
            $query->where($db->quoteName('folder') . ' = ' . $db->quote( $group ));

            $db->setQuery($query);

            return $db->loadObject();
        }


        public function abort($msg = null, $type = null){
            if ($msg) {
				Factory::getApplication()->enqueueMessage($msg, 'warning');
            }
            foreach ($this->packages as $package) {
                $package['installer']->abort(null, $type);
            }
        }
    }
}
